package br.javali.editor.io;

import java.io.*;

/** Abstrao de um disco para armazenamento de dados. */
public class Disco implements Persistencia {
    
    /** 
     * Singleton 
     */
    private static Persistencia instancia;
    private Disco() {}
    
    /**
     * Retorna uma nica instncia do objeto definido por esta classe.
     * @return instncia desta classe.
     */
    public static Persistencia getInstance() {
        if (instancia == null) {
            instancia = new Disco();
        }
        return instancia;
    }
    
    /**
     * Implementao da interface Persistencia.
     * L um arquivo do disco.
     * @param nomeDoArquivo Nome do arquivo que ser lido.
     * @return Texto contido no arquivo
     */
    public String load(String nomeDoArquivo) throws IOException {
        File arquivo = new File(nomeDoArquivo);
        BufferedReader reader = new BufferedReader(new FileReader(arquivo));
        String linha = "";
        StringBuffer buffer = new StringBuffer(8192);
        while ( (linha = reader.readLine()) != null) {
            buffer.append(linha + "\n");
        }
        reader.close();
        return buffer.toString();    
    }
    
    /**
     * Implementao da interface Persistencia.
     * Grava um arquivo no disco.
     * @param texto ATexto a ser gravado.
     * @param nomeDoArquivo Nome do arquivo que que ser gravado ou criado contendo o texto.
     */
    public void save(String texto, String nomeDoArquivo) throws IOException {
        File arquivo = new File(nomeDoArquivo);
        PrintWriter writer = new PrintWriter(new FileWriter(arquivo));
        BufferedReader reader = new BufferedReader(new StringReader(texto));
        String linha = "";
        while ( (linha = reader.readLine()) != null) {
            writer.println(linha);
        }
        writer.close();
    }
}
