package hello.jsp;

import javax.servlet.*;
import javax.servlet.http.*; 

import org.apache.cactus.*;
import junit.framework.*;
import org.apache.struts.action.*;

import org.xml.sax.SAXException;
import java.io.IOException;

/**
 * Cactus test for the front controller. We only test the extension. We assume the
 * Struts framework and the configuration files are working correctly. ToDo:
 * Test the effects of struts-config.xml and if the pages were correctly
 * redirected to their views, if the forms were filled it, etc.
 * $Revision: 1.4 $ $Date: 2002/10/06 22:46:47 $ (helder@acm.org).
 */
public class MessageActionServletTest extends ServletTestCase {
    
    public MessageActionServletTest(String name) {
        super(name);
    }
    public static Test suite() {
        return new TestSuite(MessageActionServletTest.class);
    }
    
    private String testMessage = "This is a Web Message.";

    public void beginServiceNewMessage(WebRequest clientRequest) {
        // This is failing at the service method under Cactus 1.3 (I don't know why)
        clientRequest.addParameter("message", testMessage); 
    }
    
    public void beginServiceShowLastMessage(WebRequest clientRequest) {
        // nothing to do
    }
    
    public void beginServiceShowMessages(WebRequest clientRequest) {
        // nothing to do
    }

    /////////////// Server set up and fixture /////////////////////////////
    private MessageActionServlet servlet;
    private String messageDir = "/tmp/test";
    
    public void setUp() throws ServletException {
        config.setInitParameter("message-dir", messageDir);
        config.setInitParameter("config", "/WEB-INF/struts-config.xml");
        config.setInitParameter("debug", "5");
        servlet = new MessageActionServlet();
        servlet.init(this.config);
        // Required Struts attribute
        request.setAttribute("javax.servlet.include.servlet_path", "");
    }
    
    /////////////// Server-side tests /////////////////////////////////////
    /**
     * Not much to do since most of the testing occurs in action objects.
     */
    public void _testService() throws ServletException, IOException {
        // Set locale to english so we can match the views with expected words (see endXXX())
        request.getSession().setAttribute( Action.LOCALE_KEY, java.util.Locale.ENGLISH );
        servlet.service(request, response);
        // MessageDir should have been set
        MessageBeanDAO dao = (MessageBeanDAO)request.getAttribute("dao");
        assertEquals(messageDir, dao.getLocator());
    }
    
    public void testServiceNewMessage() throws ServletException, IOException { 
        // Struts action servlet expects command in path info
        request.setAttribute("javax.servlet.include.path_info", "/newMessage");
        _testService(); 
    }
    
    public void testServiceShowLastMessage() throws ServletException, IOException  { 
        // Struts action servlet expects command in path info
        request.setAttribute("javax.servlet.include.path_info", "/showLastMessage");
        _testService(); 
    }
    
    public void testServiceShowMessages() throws ServletException, IOException  { 
        // Struts action servlet expects command in path info
        request.setAttribute("javax.servlet.include.path_info", "/showAllMessages"); 
        _testService(); 
    }
    
    /////////////////// Interface tests ///////////////////////////////////
    /**
     * Checking only if the page is the correct one!. These tests may fail if resource bundle for
     * the english language is changed.
     */
    public void endServiceNewMessage(com.meterware.httpunit.WebResponse clientResponse) throws SAXException, IOException {
        // Same as endServiceShowLastMessage()
        endServiceShowLastMessage(clientResponse);
    }
    
    /**
     * Checking only if the page is the correct one!. Todo: check if the table is present and has only one row.
     */
    public void endServiceShowLastMessage(com.meterware.httpunit.WebResponse clientResponse) throws SAXException, IOException {
        String pageTitle = clientResponse.getTitle().toLowerCase();
        assertTrue("Wrong page: " + pageTitle, pageTitle.indexOf("last") >= 0);
        String responseString = clientResponse.getText();
        assertTrue("Test Message is not in result page!", responseString.indexOf(testMessage) >= 0);
    }
    
    /**
     * Checking only if the page is the correct one!. To do: check if the number of table rows corresponds
     * to the number of files available. Check if testMessage is among the messages.
     */
    public void endServiceShowMessages(com.meterware.httpunit.WebResponse clientResponse) throws SAXException, IOException {
        String pageTitle = clientResponse.getTitle().toLowerCase();;
        assertTrue("Wrong page: " + pageTitle, pageTitle.indexOf("messages") >= 0);
        String responseString = clientResponse.getText();
        assertTrue("Test Message is not in result page!", responseString.indexOf(testMessage) >= 0);
    }
    
}
